# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nucleus/protos/reads.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nucleus.protos import cigar_pb2 as nucleus_dot_protos_dot_cigar__pb2
from nucleus.protos import position_pb2 as nucleus_dot_protos_dot_position__pb2
from nucleus.protos import reference_pb2 as nucleus_dot_protos_dot_reference__pb2
from nucleus.protos import struct_pb2 as nucleus_dot_protos_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='nucleus/protos/reads.proto',
  package='nucleus.genomics.v1',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1anucleus/protos/reads.proto\x12\x13nucleus.genomics.v1\x1a\x1anucleus/protos/cigar.proto\x1a\x1dnucleus/protos/position.proto\x1a\x1enucleus/protos/reference.proto\x1a\x1bnucleus/protos/struct.proto\"\x8a\x01\n\x0fLinearAlignment\x12/\n\x08position\x18\x01 \x01(\x0b\x32\x1d.nucleus.genomics.v1.Position\x12\x17\n\x0fmapping_quality\x18\x02 \x01(\x05\x12-\n\x05\x63igar\x18\x03 \x03(\x0b\x32\x1e.nucleus.genomics.v1.CigarUnit\"\xe0\x04\n\x04Read\x12\n\n\x02id\x18\x01 \x01(\t\x12\x15\n\rread_group_id\x18\x02 \x01(\t\x12\x19\n\x11read_group_set_id\x18\x03 \x01(\t\x12\x15\n\rfragment_name\x18\x04 \x01(\t\x12\x18\n\x10proper_placement\x18\x05 \x01(\x08\x12\x1a\n\x12\x64uplicate_fragment\x18\x06 \x01(\x08\x12\x17\n\x0f\x66ragment_length\x18\x07 \x01(\x05\x12\x13\n\x0bread_number\x18\x08 \x01(\x05\x12\x14\n\x0cnumber_reads\x18\t \x01(\x05\x12$\n\x1c\x66\x61iled_vendor_quality_checks\x18\n \x01(\x08\x12\x37\n\talignment\x18\x0b \x01(\x0b\x32$.nucleus.genomics.v1.LinearAlignment\x12\x1b\n\x13secondary_alignment\x18\x0c \x01(\x08\x12\x1f\n\x17supplementary_alignment\x18\r \x01(\x08\x12\x18\n\x10\x61ligned_sequence\x18\x0e \x01(\t\x12\x17\n\x0f\x61ligned_quality\x18\x0f \x03(\x05\x12\x39\n\x12next_mate_position\x18\x10 \x01(\x0b\x32\x1d.nucleus.genomics.v1.Position\x12\x31\n\x04info\x18\x11 \x03(\x0b\x32#.nucleus.genomics.v1.Read.InfoEntry\x1aK\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.nucleus.genomics.v1.ListValue:\x02\x38\x01\"\xe1\x03\n\tSamHeader\x12\x16\n\x0e\x66ormat_version\x18\x01 \x01(\t\x12\x42\n\rsorting_order\x18\x02 \x01(\x0e\x32+.nucleus.genomics.v1.SamHeader.SortingOrder\x12L\n\x12\x61lignment_grouping\x18\x03 \x01(\x0e\x32\x30.nucleus.genomics.v1.SamHeader.AlignmentGrouping\x12\x30\n\x07\x63ontigs\x18\x04 \x03(\x0b\x32\x1f.nucleus.genomics.v1.ContigInfo\x12\x33\n\x0bread_groups\x18\x05 \x03(\x0b\x32\x1e.nucleus.genomics.v1.ReadGroup\x12.\n\x08programs\x18\x06 \x03(\x0b\x32\x1c.nucleus.genomics.v1.Program\x12\x10\n\x08\x63omments\x18\x07 \x03(\t\"H\n\x0cSortingOrder\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08UNSORTED\x10\x01\x12\r\n\tQUERYNAME\x10\x02\x12\x0e\n\nCOORDINATE\x10\x03\"7\n\x11\x41lignmentGrouping\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05QUERY\x10\x01\x12\r\n\tREFERENCE\x10\x02\"\x9d\x02\n\tReadGroup\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x19\n\x11sequencing_center\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61te\x18\x04 \x01(\t\x12\x12\n\nflow_order\x18\x05 \x01(\t\x12\x14\n\x0ckey_sequence\x18\x06 \x01(\t\x12\x12\n\nlibrary_id\x18\x07 \x01(\t\x12\x13\n\x0bprogram_ids\x18\x08 \x03(\t\x12\x1d\n\x15predicted_insert_size\x18\t \x01(\x05\x12\x10\n\x08platform\x18\n \x01(\t\x12\x16\n\x0eplatform_model\x18\x0b \x01(\t\x12\x15\n\rplatform_unit\x18\x0c \x01(\t\x12\x11\n\tsample_id\x18\r \x01(\t\"x\n\x07Program\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x14\n\x0c\x63ommand_line\x18\x01 \x01(\t\x12\x17\n\x0fprev_program_id\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x0f\n\x07version\x18\x05 \x01(\t\"\xf0\x02\n\x10SamReaderOptions\x12@\n\x11read_requirements\x18\x01 \x01(\x0b\x32%.nucleus.genomics.v1.ReadRequirements\x12R\n\x12\x61ux_field_handling\x18\x03 \x01(\x0e\x32\x36.nucleus.genomics.v1.SamReaderOptions.AuxFieldHandling\x12\x16\n\x0ehts_block_size\x18\x04 \x01(\x03\x12\x1b\n\x13\x64ownsample_fraction\x18\x05 \x01(\x02\x12\x13\n\x0brandom_seed\x18\x06 \x01(\x03\x12(\n use_original_base_quality_scores\x18\n \x01(\x08\"R\n\x10\x41uxFieldHandling\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x13\n\x0fSKIP_AUX_FIELDS\x10\x01\x12\x18\n\x14PARSE_ALL_AUX_FIELDS\x10\x02\"\xa7\x03\n\x10ReadRequirements\x12\x17\n\x0fkeep_duplicates\x18\x01 \x01(\x08\x12)\n!keep_failed_vendor_quality_checks\x18\x02 \x01(\x08\x12!\n\x19keep_secondary_alignments\x18\x03 \x01(\x08\x12%\n\x1dkeep_supplementary_alignments\x18\x04 \x01(\x08\x12\x16\n\x0ekeep_unaligned\x18\x05 \x01(\x08\x12\x1e\n\x16keep_improperly_placed\x18\x06 \x01(\x08\x12\x1b\n\x13min_mapping_quality\x18\x07 \x01(\x05\x12\x18\n\x10min_base_quality\x18\x08 \x01(\x05\x12W\n\x15min_base_quality_mode\x18\t \x01(\x0e\x32\x38.nucleus.genomics.v1.ReadRequirements.MinBaseQualityMode\"=\n\x12MinBaseQualityMode\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x16\n\x12\x45NFORCED_BY_CLIENT\x10\x01\x62\x06proto3')
  ,
  dependencies=[nucleus_dot_protos_dot_cigar__pb2.DESCRIPTOR,nucleus_dot_protos_dot_position__pb2.DESCRIPTOR,nucleus_dot_protos_dot_reference__pb2.DESCRIPTOR,nucleus_dot_protos_dot_struct__pb2.DESCRIPTOR,])



_SAMHEADER_SORTINGORDER = _descriptor.EnumDescriptor(
  name='SortingOrder',
  full_name='nucleus.genomics.v1.SamHeader.SortingOrder',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UNSORTED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QUERYNAME', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COORDINATE', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1276,
  serialized_end=1348,
)
_sym_db.RegisterEnumDescriptor(_SAMHEADER_SORTINGORDER)

_SAMHEADER_ALIGNMENTGROUPING = _descriptor.EnumDescriptor(
  name='AlignmentGrouping',
  full_name='nucleus.genomics.v1.SamHeader.AlignmentGrouping',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='QUERY', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REFERENCE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1350,
  serialized_end=1405,
)
_sym_db.RegisterEnumDescriptor(_SAMHEADER_ALIGNMENTGROUPING)

_SAMREADEROPTIONS_AUXFIELDHANDLING = _descriptor.EnumDescriptor(
  name='AuxFieldHandling',
  full_name='nucleus.genomics.v1.SamReaderOptions.AuxFieldHandling',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SKIP_AUX_FIELDS', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PARSE_ALL_AUX_FIELDS', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2104,
  serialized_end=2186,
)
_sym_db.RegisterEnumDescriptor(_SAMREADEROPTIONS_AUXFIELDHANDLING)

_READREQUIREMENTS_MINBASEQUALITYMODE = _descriptor.EnumDescriptor(
  name='MinBaseQualityMode',
  full_name='nucleus.genomics.v1.ReadRequirements.MinBaseQualityMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ENFORCED_BY_CLIENT', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2551,
  serialized_end=2612,
)
_sym_db.RegisterEnumDescriptor(_READREQUIREMENTS_MINBASEQUALITYMODE)


_LINEARALIGNMENT = _descriptor.Descriptor(
  name='LinearAlignment',
  full_name='nucleus.genomics.v1.LinearAlignment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='position', full_name='nucleus.genomics.v1.LinearAlignment.position', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mapping_quality', full_name='nucleus.genomics.v1.LinearAlignment.mapping_quality', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cigar', full_name='nucleus.genomics.v1.LinearAlignment.cigar', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=172,
  serialized_end=310,
)


_READ_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='nucleus.genomics.v1.Read.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='nucleus.genomics.v1.Read.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='nucleus.genomics.v1.Read.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=846,
  serialized_end=921,
)

_READ = _descriptor.Descriptor(
  name='Read',
  full_name='nucleus.genomics.v1.Read',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='nucleus.genomics.v1.Read.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='read_group_id', full_name='nucleus.genomics.v1.Read.read_group_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='read_group_set_id', full_name='nucleus.genomics.v1.Read.read_group_set_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fragment_name', full_name='nucleus.genomics.v1.Read.fragment_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proper_placement', full_name='nucleus.genomics.v1.Read.proper_placement', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='duplicate_fragment', full_name='nucleus.genomics.v1.Read.duplicate_fragment', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fragment_length', full_name='nucleus.genomics.v1.Read.fragment_length', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='read_number', full_name='nucleus.genomics.v1.Read.read_number', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='number_reads', full_name='nucleus.genomics.v1.Read.number_reads', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='failed_vendor_quality_checks', full_name='nucleus.genomics.v1.Read.failed_vendor_quality_checks', index=9,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alignment', full_name='nucleus.genomics.v1.Read.alignment', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='secondary_alignment', full_name='nucleus.genomics.v1.Read.secondary_alignment', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='supplementary_alignment', full_name='nucleus.genomics.v1.Read.supplementary_alignment', index=12,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='aligned_sequence', full_name='nucleus.genomics.v1.Read.aligned_sequence', index=13,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='aligned_quality', full_name='nucleus.genomics.v1.Read.aligned_quality', index=14,
      number=15, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='next_mate_position', full_name='nucleus.genomics.v1.Read.next_mate_position', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='nucleus.genomics.v1.Read.info', index=16,
      number=17, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_READ_INFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=313,
  serialized_end=921,
)


_SAMHEADER = _descriptor.Descriptor(
  name='SamHeader',
  full_name='nucleus.genomics.v1.SamHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='format_version', full_name='nucleus.genomics.v1.SamHeader.format_version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sorting_order', full_name='nucleus.genomics.v1.SamHeader.sorting_order', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alignment_grouping', full_name='nucleus.genomics.v1.SamHeader.alignment_grouping', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contigs', full_name='nucleus.genomics.v1.SamHeader.contigs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='read_groups', full_name='nucleus.genomics.v1.SamHeader.read_groups', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='programs', full_name='nucleus.genomics.v1.SamHeader.programs', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='comments', full_name='nucleus.genomics.v1.SamHeader.comments', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SAMHEADER_SORTINGORDER,
    _SAMHEADER_ALIGNMENTGROUPING,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=924,
  serialized_end=1405,
)


_READGROUP = _descriptor.Descriptor(
  name='ReadGroup',
  full_name='nucleus.genomics.v1.ReadGroup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='nucleus.genomics.v1.ReadGroup.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sequencing_center', full_name='nucleus.genomics.v1.ReadGroup.sequencing_center', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='nucleus.genomics.v1.ReadGroup.description', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='date', full_name='nucleus.genomics.v1.ReadGroup.date', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flow_order', full_name='nucleus.genomics.v1.ReadGroup.flow_order', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key_sequence', full_name='nucleus.genomics.v1.ReadGroup.key_sequence', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='library_id', full_name='nucleus.genomics.v1.ReadGroup.library_id', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='program_ids', full_name='nucleus.genomics.v1.ReadGroup.program_ids', index=7,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='predicted_insert_size', full_name='nucleus.genomics.v1.ReadGroup.predicted_insert_size', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform', full_name='nucleus.genomics.v1.ReadGroup.platform', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform_model', full_name='nucleus.genomics.v1.ReadGroup.platform_model', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform_unit', full_name='nucleus.genomics.v1.ReadGroup.platform_unit', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sample_id', full_name='nucleus.genomics.v1.ReadGroup.sample_id', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1408,
  serialized_end=1693,
)


_PROGRAM = _descriptor.Descriptor(
  name='Program',
  full_name='nucleus.genomics.v1.Program',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='nucleus.genomics.v1.Program.id', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='nucleus.genomics.v1.Program.name', index=1,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='command_line', full_name='nucleus.genomics.v1.Program.command_line', index=2,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prev_program_id', full_name='nucleus.genomics.v1.Program.prev_program_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='nucleus.genomics.v1.Program.description', index=4,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='nucleus.genomics.v1.Program.version', index=5,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1695,
  serialized_end=1815,
)


_SAMREADEROPTIONS = _descriptor.Descriptor(
  name='SamReaderOptions',
  full_name='nucleus.genomics.v1.SamReaderOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='read_requirements', full_name='nucleus.genomics.v1.SamReaderOptions.read_requirements', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='aux_field_handling', full_name='nucleus.genomics.v1.SamReaderOptions.aux_field_handling', index=1,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hts_block_size', full_name='nucleus.genomics.v1.SamReaderOptions.hts_block_size', index=2,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='downsample_fraction', full_name='nucleus.genomics.v1.SamReaderOptions.downsample_fraction', index=3,
      number=5, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='random_seed', full_name='nucleus.genomics.v1.SamReaderOptions.random_seed', index=4,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='use_original_base_quality_scores', full_name='nucleus.genomics.v1.SamReaderOptions.use_original_base_quality_scores', index=5,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SAMREADEROPTIONS_AUXFIELDHANDLING,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1818,
  serialized_end=2186,
)


_READREQUIREMENTS = _descriptor.Descriptor(
  name='ReadRequirements',
  full_name='nucleus.genomics.v1.ReadRequirements',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='keep_duplicates', full_name='nucleus.genomics.v1.ReadRequirements.keep_duplicates', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keep_failed_vendor_quality_checks', full_name='nucleus.genomics.v1.ReadRequirements.keep_failed_vendor_quality_checks', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keep_secondary_alignments', full_name='nucleus.genomics.v1.ReadRequirements.keep_secondary_alignments', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keep_supplementary_alignments', full_name='nucleus.genomics.v1.ReadRequirements.keep_supplementary_alignments', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keep_unaligned', full_name='nucleus.genomics.v1.ReadRequirements.keep_unaligned', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keep_improperly_placed', full_name='nucleus.genomics.v1.ReadRequirements.keep_improperly_placed', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='min_mapping_quality', full_name='nucleus.genomics.v1.ReadRequirements.min_mapping_quality', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='min_base_quality', full_name='nucleus.genomics.v1.ReadRequirements.min_base_quality', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='min_base_quality_mode', full_name='nucleus.genomics.v1.ReadRequirements.min_base_quality_mode', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _READREQUIREMENTS_MINBASEQUALITYMODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2189,
  serialized_end=2612,
)

_LINEARALIGNMENT.fields_by_name['position'].message_type = nucleus_dot_protos_dot_position__pb2._POSITION
_LINEARALIGNMENT.fields_by_name['cigar'].message_type = nucleus_dot_protos_dot_cigar__pb2._CIGARUNIT
_READ_INFOENTRY.fields_by_name['value'].message_type = nucleus_dot_protos_dot_struct__pb2._LISTVALUE
_READ_INFOENTRY.containing_type = _READ
_READ.fields_by_name['alignment'].message_type = _LINEARALIGNMENT
_READ.fields_by_name['next_mate_position'].message_type = nucleus_dot_protos_dot_position__pb2._POSITION
_READ.fields_by_name['info'].message_type = _READ_INFOENTRY
_SAMHEADER.fields_by_name['sorting_order'].enum_type = _SAMHEADER_SORTINGORDER
_SAMHEADER.fields_by_name['alignment_grouping'].enum_type = _SAMHEADER_ALIGNMENTGROUPING
_SAMHEADER.fields_by_name['contigs'].message_type = nucleus_dot_protos_dot_reference__pb2._CONTIGINFO
_SAMHEADER.fields_by_name['read_groups'].message_type = _READGROUP
_SAMHEADER.fields_by_name['programs'].message_type = _PROGRAM
_SAMHEADER_SORTINGORDER.containing_type = _SAMHEADER
_SAMHEADER_ALIGNMENTGROUPING.containing_type = _SAMHEADER
_SAMREADEROPTIONS.fields_by_name['read_requirements'].message_type = _READREQUIREMENTS
_SAMREADEROPTIONS.fields_by_name['aux_field_handling'].enum_type = _SAMREADEROPTIONS_AUXFIELDHANDLING
_SAMREADEROPTIONS_AUXFIELDHANDLING.containing_type = _SAMREADEROPTIONS
_READREQUIREMENTS.fields_by_name['min_base_quality_mode'].enum_type = _READREQUIREMENTS_MINBASEQUALITYMODE
_READREQUIREMENTS_MINBASEQUALITYMODE.containing_type = _READREQUIREMENTS
DESCRIPTOR.message_types_by_name['LinearAlignment'] = _LINEARALIGNMENT
DESCRIPTOR.message_types_by_name['Read'] = _READ
DESCRIPTOR.message_types_by_name['SamHeader'] = _SAMHEADER
DESCRIPTOR.message_types_by_name['ReadGroup'] = _READGROUP
DESCRIPTOR.message_types_by_name['Program'] = _PROGRAM
DESCRIPTOR.message_types_by_name['SamReaderOptions'] = _SAMREADEROPTIONS
DESCRIPTOR.message_types_by_name['ReadRequirements'] = _READREQUIREMENTS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LinearAlignment = _reflection.GeneratedProtocolMessageType('LinearAlignment', (_message.Message,), {
  'DESCRIPTOR' : _LINEARALIGNMENT,
  '__module__' : 'nucleus.protos.reads_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.LinearAlignment)
  })
_sym_db.RegisterMessage(LinearAlignment)

Read = _reflection.GeneratedProtocolMessageType('Read', (_message.Message,), {

  'InfoEntry' : _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), {
    'DESCRIPTOR' : _READ_INFOENTRY,
    '__module__' : 'nucleus.protos.reads_pb2'
    # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.Read.InfoEntry)
    })
  ,
  'DESCRIPTOR' : _READ,
  '__module__' : 'nucleus.protos.reads_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.Read)
  })
_sym_db.RegisterMessage(Read)
_sym_db.RegisterMessage(Read.InfoEntry)

SamHeader = _reflection.GeneratedProtocolMessageType('SamHeader', (_message.Message,), {
  'DESCRIPTOR' : _SAMHEADER,
  '__module__' : 'nucleus.protos.reads_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.SamHeader)
  })
_sym_db.RegisterMessage(SamHeader)

ReadGroup = _reflection.GeneratedProtocolMessageType('ReadGroup', (_message.Message,), {
  'DESCRIPTOR' : _READGROUP,
  '__module__' : 'nucleus.protos.reads_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.ReadGroup)
  })
_sym_db.RegisterMessage(ReadGroup)

Program = _reflection.GeneratedProtocolMessageType('Program', (_message.Message,), {
  'DESCRIPTOR' : _PROGRAM,
  '__module__' : 'nucleus.protos.reads_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.Program)
  })
_sym_db.RegisterMessage(Program)

SamReaderOptions = _reflection.GeneratedProtocolMessageType('SamReaderOptions', (_message.Message,), {
  'DESCRIPTOR' : _SAMREADEROPTIONS,
  '__module__' : 'nucleus.protos.reads_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.SamReaderOptions)
  })
_sym_db.RegisterMessage(SamReaderOptions)

ReadRequirements = _reflection.GeneratedProtocolMessageType('ReadRequirements', (_message.Message,), {
  'DESCRIPTOR' : _READREQUIREMENTS,
  '__module__' : 'nucleus.protos.reads_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.ReadRequirements)
  })
_sym_db.RegisterMessage(ReadRequirements)


_READ_INFOENTRY._options = None
# @@protoc_insertion_point(module_scope)
