# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nucleus/protos/variants.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nucleus.protos import reference_pb2 as nucleus_dot_protos_dot_reference__pb2
from nucleus.protos import struct_pb2 as nucleus_dot_protos_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='nucleus/protos/variants.proto',
  package='nucleus.genomics.v1',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1dnucleus/protos/variants.proto\x12\x13nucleus.genomics.v1\x1a\x1enucleus/protos/reference.proto\x1a\x1bnucleus/protos/struct.proto\"\x9a\x03\n\x07Variant\x12\x16\n\x0ereference_name\x18\x0e \x01(\t\x12\r\n\x05start\x18\x10 \x01(\x03\x12\x0b\n\x03\x65nd\x18\r \x01(\x03\x12\r\n\x05names\x18\x03 \x03(\t\x12\x17\n\x0freference_bases\x18\x06 \x01(\t\x12\x17\n\x0f\x61lternate_bases\x18\x07 \x03(\t\x12\x0f\n\x07quality\x18\x08 \x01(\x01\x12\x0e\n\x06\x66ilter\x18\t \x03(\t\x12\x34\n\x04info\x18\n \x03(\x0b\x32&.nucleus.genomics.v1.Variant.InfoEntry\x12/\n\x05\x63\x61lls\x18\x0b \x03(\x0b\x32 .nucleus.genomics.v1.VariantCall\x12\x16\n\x0evariant_set_id\x18\x0f \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0f\n\x07\x63reated\x18\x0c \x01(\x03\x1aK\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.nucleus.genomics.v1.ListValue:\x02\x38\x01J\x04\x08\x01\x10\x02J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xa6\x02\n\x0bVariantCall\x12\x15\n\rcall_set_name\x18\t \x01(\t\x12\x10\n\x08genotype\x18\x07 \x03(\x05\x12\x11\n\tis_phased\x18\n \x01(\x08\x12\x10\n\x08phaseset\x18\x05 \x01(\t\x12\x1b\n\x13genotype_likelihood\x18\x06 \x03(\x01\x12\x38\n\x04info\x18\x02 \x03(\x0b\x32*.nucleus.genomics.v1.VariantCall.InfoEntry\x12\x13\n\x0b\x63\x61ll_set_id\x18\x08 \x01(\t\x1aK\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.nucleus.genomics.v1.ListValue:\x02\x38\x01J\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"\xf1\x02\n\tVcfHeader\x12\x12\n\nfileformat\x18\x01 \x01(\t\x12\x30\n\x07\x63ontigs\x18\x02 \x03(\x0b\x32\x1f.nucleus.genomics.v1.ContigInfo\x12\x33\n\x07\x66ilters\x18\x03 \x03(\x0b\x32\".nucleus.genomics.v1.VcfFilterInfo\x12+\n\x05infos\x18\x04 \x03(\x0b\x32\x1c.nucleus.genomics.v1.VcfInfo\x12\x33\n\x07\x66ormats\x18\x05 \x03(\x0b\x32\".nucleus.genomics.v1.VcfFormatInfo\x12\x14\n\x0csample_names\x18\x06 \x03(\t\x12\x42\n\x11structured_extras\x18\x08 \x03(\x0b\x32\'.nucleus.genomics.v1.VcfStructuredExtra\x12-\n\x06\x65xtras\x18\x07 \x03(\x0b\x32\x1d.nucleus.genomics.v1.VcfExtra\"0\n\rVcfFilterInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\"i\n\x07VcfInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06number\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x0e\n\x06source\x18\x05 \x01(\t\x12\x0f\n\x07version\x18\x06 \x01(\t\"N\n\rVcfFormatInfo\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06number\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\"P\n\x12VcfStructuredExtra\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x06\x66ields\x18\x02 \x03(\x0b\x32\x1d.nucleus.genomics.v1.VcfExtra\"&\n\x08VcfExtra\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\x81\x01\n\x10VcfReaderOptions\x12\x1c\n\x14\x65xcluded_info_fields\x18\x03 \x03(\t\x12\x1e\n\x16\x65xcluded_format_fields\x18\x04 \x03(\t\x12#\n\x1bstore_gl_and_pl_in_info_map\x18\x05 \x01(\x08J\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"\xcb\x01\n\x10VcfWriterOptions\x12\x1c\n\x14\x65xcluded_info_fields\x18\x07 \x03(\t\x12\x1e\n\x16\x65xcluded_format_fields\x18\x08 \x03(\t\x12\x19\n\x11round_qual_values\x18\x06 \x01(\x08\x12(\n retrieve_gl_and_pl_from_info_map\x18\t \x01(\x08\x12\x16\n\x0e\x65xclude_header\x18\n \x01(\x08J\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\x62\x06proto3')
  ,
  dependencies=[nucleus_dot_protos_dot_reference__pb2.DESCRIPTOR,nucleus_dot_protos_dot_struct__pb2.DESCRIPTOR,])




_VARIANT_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='nucleus.genomics.v1.Variant.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='nucleus.genomics.v1.Variant.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='nucleus.genomics.v1.Variant.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=433,
  serialized_end=508,
)

_VARIANT = _descriptor.Descriptor(
  name='Variant',
  full_name='nucleus.genomics.v1.Variant',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference_name', full_name='nucleus.genomics.v1.Variant.reference_name', index=0,
      number=14, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='nucleus.genomics.v1.Variant.start', index=1,
      number=16, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end', full_name='nucleus.genomics.v1.Variant.end', index=2,
      number=13, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='names', full_name='nucleus.genomics.v1.Variant.names', index=3,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reference_bases', full_name='nucleus.genomics.v1.Variant.reference_bases', index=4,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alternate_bases', full_name='nucleus.genomics.v1.Variant.alternate_bases', index=5,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='quality', full_name='nucleus.genomics.v1.Variant.quality', index=6,
      number=8, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filter', full_name='nucleus.genomics.v1.Variant.filter', index=7,
      number=9, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='nucleus.genomics.v1.Variant.info', index=8,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='calls', full_name='nucleus.genomics.v1.Variant.calls', index=9,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='variant_set_id', full_name='nucleus.genomics.v1.Variant.variant_set_id', index=10,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='id', full_name='nucleus.genomics.v1.Variant.id', index=11,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created', full_name='nucleus.genomics.v1.Variant.created', index=12,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_VARIANT_INFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=116,
  serialized_end=526,
)


_VARIANTCALL_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='nucleus.genomics.v1.VariantCall.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='nucleus.genomics.v1.VariantCall.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='nucleus.genomics.v1.VariantCall.InfoEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=433,
  serialized_end=508,
)

_VARIANTCALL = _descriptor.Descriptor(
  name='VariantCall',
  full_name='nucleus.genomics.v1.VariantCall',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='call_set_name', full_name='nucleus.genomics.v1.VariantCall.call_set_name', index=0,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genotype', full_name='nucleus.genomics.v1.VariantCall.genotype', index=1,
      number=7, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_phased', full_name='nucleus.genomics.v1.VariantCall.is_phased', index=2,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='phaseset', full_name='nucleus.genomics.v1.VariantCall.phaseset', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genotype_likelihood', full_name='nucleus.genomics.v1.VariantCall.genotype_likelihood', index=4,
      number=6, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='nucleus.genomics.v1.VariantCall.info', index=5,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='call_set_id', full_name='nucleus.genomics.v1.VariantCall.call_set_id', index=6,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_VARIANTCALL_INFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=529,
  serialized_end=823,
)


_VCFHEADER = _descriptor.Descriptor(
  name='VcfHeader',
  full_name='nucleus.genomics.v1.VcfHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fileformat', full_name='nucleus.genomics.v1.VcfHeader.fileformat', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contigs', full_name='nucleus.genomics.v1.VcfHeader.contigs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='filters', full_name='nucleus.genomics.v1.VcfHeader.filters', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='infos', full_name='nucleus.genomics.v1.VcfHeader.infos', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='formats', full_name='nucleus.genomics.v1.VcfHeader.formats', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sample_names', full_name='nucleus.genomics.v1.VcfHeader.sample_names', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='structured_extras', full_name='nucleus.genomics.v1.VcfHeader.structured_extras', index=6,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extras', full_name='nucleus.genomics.v1.VcfHeader.extras', index=7,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=826,
  serialized_end=1195,
)


_VCFFILTERINFO = _descriptor.Descriptor(
  name='VcfFilterInfo',
  full_name='nucleus.genomics.v1.VcfFilterInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='nucleus.genomics.v1.VcfFilterInfo.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='nucleus.genomics.v1.VcfFilterInfo.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1197,
  serialized_end=1245,
)


_VCFINFO = _descriptor.Descriptor(
  name='VcfInfo',
  full_name='nucleus.genomics.v1.VcfInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='nucleus.genomics.v1.VcfInfo.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='number', full_name='nucleus.genomics.v1.VcfInfo.number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='nucleus.genomics.v1.VcfInfo.type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='nucleus.genomics.v1.VcfInfo.description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='nucleus.genomics.v1.VcfInfo.source', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='nucleus.genomics.v1.VcfInfo.version', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1247,
  serialized_end=1352,
)


_VCFFORMATINFO = _descriptor.Descriptor(
  name='VcfFormatInfo',
  full_name='nucleus.genomics.v1.VcfFormatInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='nucleus.genomics.v1.VcfFormatInfo.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='number', full_name='nucleus.genomics.v1.VcfFormatInfo.number', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='nucleus.genomics.v1.VcfFormatInfo.type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='nucleus.genomics.v1.VcfFormatInfo.description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1354,
  serialized_end=1432,
)


_VCFSTRUCTUREDEXTRA = _descriptor.Descriptor(
  name='VcfStructuredExtra',
  full_name='nucleus.genomics.v1.VcfStructuredExtra',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='nucleus.genomics.v1.VcfStructuredExtra.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fields', full_name='nucleus.genomics.v1.VcfStructuredExtra.fields', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1434,
  serialized_end=1514,
)


_VCFEXTRA = _descriptor.Descriptor(
  name='VcfExtra',
  full_name='nucleus.genomics.v1.VcfExtra',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='nucleus.genomics.v1.VcfExtra.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='nucleus.genomics.v1.VcfExtra.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1516,
  serialized_end=1554,
)


_VCFREADEROPTIONS = _descriptor.Descriptor(
  name='VcfReaderOptions',
  full_name='nucleus.genomics.v1.VcfReaderOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='excluded_info_fields', full_name='nucleus.genomics.v1.VcfReaderOptions.excluded_info_fields', index=0,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excluded_format_fields', full_name='nucleus.genomics.v1.VcfReaderOptions.excluded_format_fields', index=1,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='store_gl_and_pl_in_info_map', full_name='nucleus.genomics.v1.VcfReaderOptions.store_gl_and_pl_in_info_map', index=2,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1557,
  serialized_end=1686,
)


_VCFWRITEROPTIONS = _descriptor.Descriptor(
  name='VcfWriterOptions',
  full_name='nucleus.genomics.v1.VcfWriterOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='excluded_info_fields', full_name='nucleus.genomics.v1.VcfWriterOptions.excluded_info_fields', index=0,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excluded_format_fields', full_name='nucleus.genomics.v1.VcfWriterOptions.excluded_format_fields', index=1,
      number=8, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='round_qual_values', full_name='nucleus.genomics.v1.VcfWriterOptions.round_qual_values', index=2,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='retrieve_gl_and_pl_from_info_map', full_name='nucleus.genomics.v1.VcfWriterOptions.retrieve_gl_and_pl_from_info_map', index=3,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exclude_header', full_name='nucleus.genomics.v1.VcfWriterOptions.exclude_header', index=4,
      number=10, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1689,
  serialized_end=1892,
)

_VARIANT_INFOENTRY.fields_by_name['value'].message_type = nucleus_dot_protos_dot_struct__pb2._LISTVALUE
_VARIANT_INFOENTRY.containing_type = _VARIANT
_VARIANT.fields_by_name['info'].message_type = _VARIANT_INFOENTRY
_VARIANT.fields_by_name['calls'].message_type = _VARIANTCALL
_VARIANTCALL_INFOENTRY.fields_by_name['value'].message_type = nucleus_dot_protos_dot_struct__pb2._LISTVALUE
_VARIANTCALL_INFOENTRY.containing_type = _VARIANTCALL
_VARIANTCALL.fields_by_name['info'].message_type = _VARIANTCALL_INFOENTRY
_VCFHEADER.fields_by_name['contigs'].message_type = nucleus_dot_protos_dot_reference__pb2._CONTIGINFO
_VCFHEADER.fields_by_name['filters'].message_type = _VCFFILTERINFO
_VCFHEADER.fields_by_name['infos'].message_type = _VCFINFO
_VCFHEADER.fields_by_name['formats'].message_type = _VCFFORMATINFO
_VCFHEADER.fields_by_name['structured_extras'].message_type = _VCFSTRUCTUREDEXTRA
_VCFHEADER.fields_by_name['extras'].message_type = _VCFEXTRA
_VCFSTRUCTUREDEXTRA.fields_by_name['fields'].message_type = _VCFEXTRA
DESCRIPTOR.message_types_by_name['Variant'] = _VARIANT
DESCRIPTOR.message_types_by_name['VariantCall'] = _VARIANTCALL
DESCRIPTOR.message_types_by_name['VcfHeader'] = _VCFHEADER
DESCRIPTOR.message_types_by_name['VcfFilterInfo'] = _VCFFILTERINFO
DESCRIPTOR.message_types_by_name['VcfInfo'] = _VCFINFO
DESCRIPTOR.message_types_by_name['VcfFormatInfo'] = _VCFFORMATINFO
DESCRIPTOR.message_types_by_name['VcfStructuredExtra'] = _VCFSTRUCTUREDEXTRA
DESCRIPTOR.message_types_by_name['VcfExtra'] = _VCFEXTRA
DESCRIPTOR.message_types_by_name['VcfReaderOptions'] = _VCFREADEROPTIONS
DESCRIPTOR.message_types_by_name['VcfWriterOptions'] = _VCFWRITEROPTIONS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Variant = _reflection.GeneratedProtocolMessageType('Variant', (_message.Message,), {

  'InfoEntry' : _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), {
    'DESCRIPTOR' : _VARIANT_INFOENTRY,
    '__module__' : 'nucleus.protos.variants_pb2'
    # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.Variant.InfoEntry)
    })
  ,
  'DESCRIPTOR' : _VARIANT,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.Variant)
  })
_sym_db.RegisterMessage(Variant)
_sym_db.RegisterMessage(Variant.InfoEntry)

VariantCall = _reflection.GeneratedProtocolMessageType('VariantCall', (_message.Message,), {

  'InfoEntry' : _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), {
    'DESCRIPTOR' : _VARIANTCALL_INFOENTRY,
    '__module__' : 'nucleus.protos.variants_pb2'
    # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VariantCall.InfoEntry)
    })
  ,
  'DESCRIPTOR' : _VARIANTCALL,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VariantCall)
  })
_sym_db.RegisterMessage(VariantCall)
_sym_db.RegisterMessage(VariantCall.InfoEntry)

VcfHeader = _reflection.GeneratedProtocolMessageType('VcfHeader', (_message.Message,), {
  'DESCRIPTOR' : _VCFHEADER,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfHeader)
  })
_sym_db.RegisterMessage(VcfHeader)

VcfFilterInfo = _reflection.GeneratedProtocolMessageType('VcfFilterInfo', (_message.Message,), {
  'DESCRIPTOR' : _VCFFILTERINFO,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfFilterInfo)
  })
_sym_db.RegisterMessage(VcfFilterInfo)

VcfInfo = _reflection.GeneratedProtocolMessageType('VcfInfo', (_message.Message,), {
  'DESCRIPTOR' : _VCFINFO,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfInfo)
  })
_sym_db.RegisterMessage(VcfInfo)

VcfFormatInfo = _reflection.GeneratedProtocolMessageType('VcfFormatInfo', (_message.Message,), {
  'DESCRIPTOR' : _VCFFORMATINFO,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfFormatInfo)
  })
_sym_db.RegisterMessage(VcfFormatInfo)

VcfStructuredExtra = _reflection.GeneratedProtocolMessageType('VcfStructuredExtra', (_message.Message,), {
  'DESCRIPTOR' : _VCFSTRUCTUREDEXTRA,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfStructuredExtra)
  })
_sym_db.RegisterMessage(VcfStructuredExtra)

VcfExtra = _reflection.GeneratedProtocolMessageType('VcfExtra', (_message.Message,), {
  'DESCRIPTOR' : _VCFEXTRA,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfExtra)
  })
_sym_db.RegisterMessage(VcfExtra)

VcfReaderOptions = _reflection.GeneratedProtocolMessageType('VcfReaderOptions', (_message.Message,), {
  'DESCRIPTOR' : _VCFREADEROPTIONS,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfReaderOptions)
  })
_sym_db.RegisterMessage(VcfReaderOptions)

VcfWriterOptions = _reflection.GeneratedProtocolMessageType('VcfWriterOptions', (_message.Message,), {
  'DESCRIPTOR' : _VCFWRITEROPTIONS,
  '__module__' : 'nucleus.protos.variants_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.VcfWriterOptions)
  })
_sym_db.RegisterMessage(VcfWriterOptions)


_VARIANT_INFOENTRY._options = None
_VARIANTCALL_INFOENTRY._options = None
# @@protoc_insertion_point(module_scope)
