# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nucleus/protos/bed.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='nucleus/protos/bed.proto',
  package='nucleus.genomics.v1',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x18nucleus/protos/bed.proto\x12\x13nucleus.genomics.v1\"\xce\x02\n\tBedRecord\x12\x16\n\x0ereference_name\x18\x01 \x01(\t\x12\r\n\x05start\x18\x02 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x03\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\r\n\x05score\x18\x05 \x01(\x01\x12\x35\n\x06strand\x18\x06 \x01(\x0e\x32%.nucleus.genomics.v1.BedRecord.Strand\x12\x13\n\x0bthick_start\x18\x07 \x01(\x03\x12\x11\n\tthick_end\x18\x08 \x01(\x03\x12\x10\n\x08item_rgb\x18\t \x01(\t\x12\x13\n\x0b\x62lock_count\x18\n \x01(\x05\x12\x13\n\x0b\x62lock_sizes\x18\x0b \x01(\t\x12\x14\n\x0c\x62lock_starts\x18\x0c \x01(\t\"?\n\x06Strand\x12\r\n\tNO_STRAND\x10\x00\x12\x12\n\x0e\x46ORWARD_STRAND\x10\x01\x12\x12\n\x0eREVERSE_STRAND\x10\x02\"\x1f\n\tBedHeader\x12\x12\n\nnum_fields\x18\x01 \x01(\x05\",\n\x10\x42\x65\x64ReaderOptions\x12\x12\n\nnum_fields\x18\x02 \x01(\x05J\x04\x08\x01\x10\x02\"\x12\n\x10\x42\x65\x64WriterOptionsb\x06proto3')
)



_BEDRECORD_STRAND = _descriptor.EnumDescriptor(
  name='Strand',
  full_name='nucleus.genomics.v1.BedRecord.Strand',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_STRAND', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FORWARD_STRAND', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REVERSE_STRAND', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=321,
  serialized_end=384,
)
_sym_db.RegisterEnumDescriptor(_BEDRECORD_STRAND)


_BEDRECORD = _descriptor.Descriptor(
  name='BedRecord',
  full_name='nucleus.genomics.v1.BedRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference_name', full_name='nucleus.genomics.v1.BedRecord.reference_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='nucleus.genomics.v1.BedRecord.start', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end', full_name='nucleus.genomics.v1.BedRecord.end', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='nucleus.genomics.v1.BedRecord.name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='nucleus.genomics.v1.BedRecord.score', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='strand', full_name='nucleus.genomics.v1.BedRecord.strand', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thick_start', full_name='nucleus.genomics.v1.BedRecord.thick_start', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thick_end', full_name='nucleus.genomics.v1.BedRecord.thick_end', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='item_rgb', full_name='nucleus.genomics.v1.BedRecord.item_rgb', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block_count', full_name='nucleus.genomics.v1.BedRecord.block_count', index=9,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block_sizes', full_name='nucleus.genomics.v1.BedRecord.block_sizes', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='block_starts', full_name='nucleus.genomics.v1.BedRecord.block_starts', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _BEDRECORD_STRAND,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=384,
)


_BEDHEADER = _descriptor.Descriptor(
  name='BedHeader',
  full_name='nucleus.genomics.v1.BedHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_fields', full_name='nucleus.genomics.v1.BedHeader.num_fields', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=386,
  serialized_end=417,
)


_BEDREADEROPTIONS = _descriptor.Descriptor(
  name='BedReaderOptions',
  full_name='nucleus.genomics.v1.BedReaderOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='num_fields', full_name='nucleus.genomics.v1.BedReaderOptions.num_fields', index=0,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=419,
  serialized_end=463,
)


_BEDWRITEROPTIONS = _descriptor.Descriptor(
  name='BedWriterOptions',
  full_name='nucleus.genomics.v1.BedWriterOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=465,
  serialized_end=483,
)

_BEDRECORD.fields_by_name['strand'].enum_type = _BEDRECORD_STRAND
_BEDRECORD_STRAND.containing_type = _BEDRECORD
DESCRIPTOR.message_types_by_name['BedRecord'] = _BEDRECORD
DESCRIPTOR.message_types_by_name['BedHeader'] = _BEDHEADER
DESCRIPTOR.message_types_by_name['BedReaderOptions'] = _BEDREADEROPTIONS
DESCRIPTOR.message_types_by_name['BedWriterOptions'] = _BEDWRITEROPTIONS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

BedRecord = _reflection.GeneratedProtocolMessageType('BedRecord', (_message.Message,), {
  'DESCRIPTOR' : _BEDRECORD,
  '__module__' : 'nucleus.protos.bed_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.BedRecord)
  })
_sym_db.RegisterMessage(BedRecord)

BedHeader = _reflection.GeneratedProtocolMessageType('BedHeader', (_message.Message,), {
  'DESCRIPTOR' : _BEDHEADER,
  '__module__' : 'nucleus.protos.bed_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.BedHeader)
  })
_sym_db.RegisterMessage(BedHeader)

BedReaderOptions = _reflection.GeneratedProtocolMessageType('BedReaderOptions', (_message.Message,), {
  'DESCRIPTOR' : _BEDREADEROPTIONS,
  '__module__' : 'nucleus.protos.bed_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.BedReaderOptions)
  })
_sym_db.RegisterMessage(BedReaderOptions)

BedWriterOptions = _reflection.GeneratedProtocolMessageType('BedWriterOptions', (_message.Message,), {
  'DESCRIPTOR' : _BEDWRITEROPTIONS,
  '__module__' : 'nucleus.protos.bed_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.BedWriterOptions)
  })
_sym_db.RegisterMessage(BedWriterOptions)


# @@protoc_insertion_point(module_scope)
