# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nucleus/protos/gff.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nucleus.protos import range_pb2 as nucleus_dot_protos_dot_range__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='nucleus/protos/gff.proto',
  package='nucleus.genomics.v1',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x18nucleus/protos/gff.proto\x12\x13nucleus.genomics.v1\x1a\x1anucleus/protos/range.proto\"\xea\x02\n\tGffRecord\x12)\n\x05range\x18\x01 \x01(\x0b\x32\x1a.nucleus.genomics.v1.Range\x12\x0e\n\x06source\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\r\n\x05score\x18\x04 \x01(\x01\x12\x35\n\x06strand\x18\x05 \x01(\x0e\x32%.nucleus.genomics.v1.GffRecord.Strand\x12\r\n\x05phase\x18\x06 \x01(\x05\x12\x42\n\nattributes\x18\x07 \x03(\x0b\x32..nucleus.genomics.v1.GffRecord.AttributesEntry\x1a\x31\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"H\n\x06Strand\x12\x16\n\x12UNSPECIFIED_STRAND\x10\x00\x12\x12\n\x0e\x46ORWARD_STRAND\x10\x01\x12\x12\n\x0eREVERSE_STRAND\x10\x02\"\xf5\x03\n\tGffHeader\x12\x13\n\x0bgff_version\x18\x01 \x01(\t\x12\x34\n\x10sequence_regions\x18\x02 \x03(\x0b\x32\x1a.nucleus.genomics.v1.Range\x12L\n\x12\x66\x65\x61ture_ontologies\x18\x03 \x03(\x0b\x32\x30.nucleus.genomics.v1.GffHeader.OntologyDirective\x12N\n\x14\x61ttribute_ontologies\x18\x04 \x03(\x0b\x32\x30.nucleus.genomics.v1.GffHeader.OntologyDirective\x12K\n\x11source_ontologies\x18\x05 \x03(\x0b\x32\x30.nucleus.genomics.v1.GffHeader.OntologyDirective\x12\x0f\n\x07species\x18\x06 \x01(\t\x12I\n\x0cgenome_build\x18\x07 \x01(\x0b\x32\x33.nucleus.genomics.v1.GffHeader.GenomeBuildDirective\x1a \n\x11OntologyDirective\x12\x0b\n\x03uri\x18\x01 \x01(\t\x1a\x34\n\x14GenomeBuildDirective\x12\x0e\n\x06source\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"\x12\n\x10GffReaderOptions\"\x12\n\x10GffWriterOptionsb\x06proto3')
  ,
  dependencies=[nucleus_dot_protos_dot_range__pb2.DESCRIPTOR,])



_GFFRECORD_STRAND = _descriptor.EnumDescriptor(
  name='Strand',
  full_name='nucleus.genomics.v1.GffRecord.Strand',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED_STRAND', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FORWARD_STRAND', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REVERSE_STRAND', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=368,
  serialized_end=440,
)
_sym_db.RegisterEnumDescriptor(_GFFRECORD_STRAND)


_GFFRECORD_ATTRIBUTESENTRY = _descriptor.Descriptor(
  name='AttributesEntry',
  full_name='nucleus.genomics.v1.GffRecord.AttributesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='nucleus.genomics.v1.GffRecord.AttributesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='nucleus.genomics.v1.GffRecord.AttributesEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=317,
  serialized_end=366,
)

_GFFRECORD = _descriptor.Descriptor(
  name='GffRecord',
  full_name='nucleus.genomics.v1.GffRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='range', full_name='nucleus.genomics.v1.GffRecord.range', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source', full_name='nucleus.genomics.v1.GffRecord.source', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='nucleus.genomics.v1.GffRecord.type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='nucleus.genomics.v1.GffRecord.score', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='strand', full_name='nucleus.genomics.v1.GffRecord.strand', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='phase', full_name='nucleus.genomics.v1.GffRecord.phase', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='nucleus.genomics.v1.GffRecord.attributes', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GFFRECORD_ATTRIBUTESENTRY, ],
  enum_types=[
    _GFFRECORD_STRAND,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=78,
  serialized_end=440,
)


_GFFHEADER_ONTOLOGYDIRECTIVE = _descriptor.Descriptor(
  name='OntologyDirective',
  full_name='nucleus.genomics.v1.GffHeader.OntologyDirective',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uri', full_name='nucleus.genomics.v1.GffHeader.OntologyDirective.uri', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=858,
  serialized_end=890,
)

_GFFHEADER_GENOMEBUILDDIRECTIVE = _descriptor.Descriptor(
  name='GenomeBuildDirective',
  full_name='nucleus.genomics.v1.GffHeader.GenomeBuildDirective',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='source', full_name='nucleus.genomics.v1.GffHeader.GenomeBuildDirective.source', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='nucleus.genomics.v1.GffHeader.GenomeBuildDirective.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=892,
  serialized_end=944,
)

_GFFHEADER = _descriptor.Descriptor(
  name='GffHeader',
  full_name='nucleus.genomics.v1.GffHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='gff_version', full_name='nucleus.genomics.v1.GffHeader.gff_version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sequence_regions', full_name='nucleus.genomics.v1.GffHeader.sequence_regions', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='feature_ontologies', full_name='nucleus.genomics.v1.GffHeader.feature_ontologies', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attribute_ontologies', full_name='nucleus.genomics.v1.GffHeader.attribute_ontologies', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='source_ontologies', full_name='nucleus.genomics.v1.GffHeader.source_ontologies', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='species', full_name='nucleus.genomics.v1.GffHeader.species', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='genome_build', full_name='nucleus.genomics.v1.GffHeader.genome_build', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_GFFHEADER_ONTOLOGYDIRECTIVE, _GFFHEADER_GENOMEBUILDDIRECTIVE, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=944,
)


_GFFREADEROPTIONS = _descriptor.Descriptor(
  name='GffReaderOptions',
  full_name='nucleus.genomics.v1.GffReaderOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=946,
  serialized_end=964,
)


_GFFWRITEROPTIONS = _descriptor.Descriptor(
  name='GffWriterOptions',
  full_name='nucleus.genomics.v1.GffWriterOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=966,
  serialized_end=984,
)

_GFFRECORD_ATTRIBUTESENTRY.containing_type = _GFFRECORD
_GFFRECORD.fields_by_name['range'].message_type = nucleus_dot_protos_dot_range__pb2._RANGE
_GFFRECORD.fields_by_name['strand'].enum_type = _GFFRECORD_STRAND
_GFFRECORD.fields_by_name['attributes'].message_type = _GFFRECORD_ATTRIBUTESENTRY
_GFFRECORD_STRAND.containing_type = _GFFRECORD
_GFFHEADER_ONTOLOGYDIRECTIVE.containing_type = _GFFHEADER
_GFFHEADER_GENOMEBUILDDIRECTIVE.containing_type = _GFFHEADER
_GFFHEADER.fields_by_name['sequence_regions'].message_type = nucleus_dot_protos_dot_range__pb2._RANGE
_GFFHEADER.fields_by_name['feature_ontologies'].message_type = _GFFHEADER_ONTOLOGYDIRECTIVE
_GFFHEADER.fields_by_name['attribute_ontologies'].message_type = _GFFHEADER_ONTOLOGYDIRECTIVE
_GFFHEADER.fields_by_name['source_ontologies'].message_type = _GFFHEADER_ONTOLOGYDIRECTIVE
_GFFHEADER.fields_by_name['genome_build'].message_type = _GFFHEADER_GENOMEBUILDDIRECTIVE
DESCRIPTOR.message_types_by_name['GffRecord'] = _GFFRECORD
DESCRIPTOR.message_types_by_name['GffHeader'] = _GFFHEADER
DESCRIPTOR.message_types_by_name['GffReaderOptions'] = _GFFREADEROPTIONS
DESCRIPTOR.message_types_by_name['GffWriterOptions'] = _GFFWRITEROPTIONS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GffRecord = _reflection.GeneratedProtocolMessageType('GffRecord', (_message.Message,), {

  'AttributesEntry' : _reflection.GeneratedProtocolMessageType('AttributesEntry', (_message.Message,), {
    'DESCRIPTOR' : _GFFRECORD_ATTRIBUTESENTRY,
    '__module__' : 'nucleus.protos.gff_pb2'
    # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.GffRecord.AttributesEntry)
    })
  ,
  'DESCRIPTOR' : _GFFRECORD,
  '__module__' : 'nucleus.protos.gff_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.GffRecord)
  })
_sym_db.RegisterMessage(GffRecord)
_sym_db.RegisterMessage(GffRecord.AttributesEntry)

GffHeader = _reflection.GeneratedProtocolMessageType('GffHeader', (_message.Message,), {

  'OntologyDirective' : _reflection.GeneratedProtocolMessageType('OntologyDirective', (_message.Message,), {
    'DESCRIPTOR' : _GFFHEADER_ONTOLOGYDIRECTIVE,
    '__module__' : 'nucleus.protos.gff_pb2'
    # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.GffHeader.OntologyDirective)
    })
  ,

  'GenomeBuildDirective' : _reflection.GeneratedProtocolMessageType('GenomeBuildDirective', (_message.Message,), {
    'DESCRIPTOR' : _GFFHEADER_GENOMEBUILDDIRECTIVE,
    '__module__' : 'nucleus.protos.gff_pb2'
    # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.GffHeader.GenomeBuildDirective)
    })
  ,
  'DESCRIPTOR' : _GFFHEADER,
  '__module__' : 'nucleus.protos.gff_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.GffHeader)
  })
_sym_db.RegisterMessage(GffHeader)
_sym_db.RegisterMessage(GffHeader.OntologyDirective)
_sym_db.RegisterMessage(GffHeader.GenomeBuildDirective)

GffReaderOptions = _reflection.GeneratedProtocolMessageType('GffReaderOptions', (_message.Message,), {
  'DESCRIPTOR' : _GFFREADEROPTIONS,
  '__module__' : 'nucleus.protos.gff_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.GffReaderOptions)
  })
_sym_db.RegisterMessage(GffReaderOptions)

GffWriterOptions = _reflection.GeneratedProtocolMessageType('GffWriterOptions', (_message.Message,), {
  'DESCRIPTOR' : _GFFWRITEROPTIONS,
  '__module__' : 'nucleus.protos.gff_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.GffWriterOptions)
  })
_sym_db.RegisterMessage(GffWriterOptions)


_GFFRECORD_ATTRIBUTESENTRY._options = None
# @@protoc_insertion_point(module_scope)
