# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nucleus/protos/fasta.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nucleus.protos import range_pb2 as nucleus_dot_protos_dot_range__pb2
from nucleus.protos import reference_pb2 as nucleus_dot_protos_dot_reference__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='nucleus/protos/fasta.proto',
  package='nucleus.genomics.v1',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1anucleus/protos/fasta.proto\x12\x13nucleus.genomics.v1\x1a\x1anucleus/protos/range.proto\x1a\x1enucleus/protos/reference.proto\"\x8d\x01\n\x0b\x46\x61staRecord\x12\x0f\n\x07\x64\x65\x66line\x18\x01 \x01(\t\x12/\n\x06\x63ontig\x18\x02 \x01(\x0b\x32\x1f.nucleus.genomics.v1.ContigInfo\x12*\n\x06region\x18\x03 \x01(\x0b\x32\x1a.nucleus.genomics.v1.Range\x12\x10\n\x08sequence\x18\x04 \x01(\t\"\xde\x01\n\x12\x46\x61staReaderOptions\x12\x16\n\x0ekeep_true_case\x18\x01 \x01(\x08\x12\x10\n\x08\x61lphabet\x18\x02 \x01(\t\x12O\n\x0f\x64\x65\x66line_parsing\x18\x03 \x01(\x0e\x32\x36.nucleus.genomics.v1.FastaReaderOptions.DeflineParsing\x12 \n\x18include_range_in_records\x18\x04 \x01(\x08\"+\n\x0e\x44\x65\x66lineParsing\x12\x08\n\x04NONE\x10\x00\x12\x0f\n\x0b\x43ONTIG_INFO\x10\x01\"\x14\n\x12\x46\x61staWriterOptionsb\x06proto3')
  ,
  dependencies=[nucleus_dot_protos_dot_range__pb2.DESCRIPTOR,nucleus_dot_protos_dot_reference__pb2.DESCRIPTOR,])



_FASTAREADEROPTIONS_DEFLINEPARSING = _descriptor.EnumDescriptor(
  name='DeflineParsing',
  full_name='nucleus.genomics.v1.FastaReaderOptions.DeflineParsing',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONTIG_INFO', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=435,
  serialized_end=478,
)
_sym_db.RegisterEnumDescriptor(_FASTAREADEROPTIONS_DEFLINEPARSING)


_FASTARECORD = _descriptor.Descriptor(
  name='FastaRecord',
  full_name='nucleus.genomics.v1.FastaRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='defline', full_name='nucleus.genomics.v1.FastaRecord.defline', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='contig', full_name='nucleus.genomics.v1.FastaRecord.contig', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='region', full_name='nucleus.genomics.v1.FastaRecord.region', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sequence', full_name='nucleus.genomics.v1.FastaRecord.sequence', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=112,
  serialized_end=253,
)


_FASTAREADEROPTIONS = _descriptor.Descriptor(
  name='FastaReaderOptions',
  full_name='nucleus.genomics.v1.FastaReaderOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='keep_true_case', full_name='nucleus.genomics.v1.FastaReaderOptions.keep_true_case', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='alphabet', full_name='nucleus.genomics.v1.FastaReaderOptions.alphabet', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='defline_parsing', full_name='nucleus.genomics.v1.FastaReaderOptions.defline_parsing', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='include_range_in_records', full_name='nucleus.genomics.v1.FastaReaderOptions.include_range_in_records', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FASTAREADEROPTIONS_DEFLINEPARSING,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=256,
  serialized_end=478,
)


_FASTAWRITEROPTIONS = _descriptor.Descriptor(
  name='FastaWriterOptions',
  full_name='nucleus.genomics.v1.FastaWriterOptions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=480,
  serialized_end=500,
)

_FASTARECORD.fields_by_name['contig'].message_type = nucleus_dot_protos_dot_reference__pb2._CONTIGINFO
_FASTARECORD.fields_by_name['region'].message_type = nucleus_dot_protos_dot_range__pb2._RANGE
_FASTAREADEROPTIONS.fields_by_name['defline_parsing'].enum_type = _FASTAREADEROPTIONS_DEFLINEPARSING
_FASTAREADEROPTIONS_DEFLINEPARSING.containing_type = _FASTAREADEROPTIONS
DESCRIPTOR.message_types_by_name['FastaRecord'] = _FASTARECORD
DESCRIPTOR.message_types_by_name['FastaReaderOptions'] = _FASTAREADEROPTIONS
DESCRIPTOR.message_types_by_name['FastaWriterOptions'] = _FASTAWRITEROPTIONS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FastaRecord = _reflection.GeneratedProtocolMessageType('FastaRecord', (_message.Message,), {
  'DESCRIPTOR' : _FASTARECORD,
  '__module__' : 'nucleus.protos.fasta_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.FastaRecord)
  })
_sym_db.RegisterMessage(FastaRecord)

FastaReaderOptions = _reflection.GeneratedProtocolMessageType('FastaReaderOptions', (_message.Message,), {
  'DESCRIPTOR' : _FASTAREADEROPTIONS,
  '__module__' : 'nucleus.protos.fasta_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.FastaReaderOptions)
  })
_sym_db.RegisterMessage(FastaReaderOptions)

FastaWriterOptions = _reflection.GeneratedProtocolMessageType('FastaWriterOptions', (_message.Message,), {
  'DESCRIPTOR' : _FASTAWRITEROPTIONS,
  '__module__' : 'nucleus.protos.fasta_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.FastaWriterOptions)
  })
_sym_db.RegisterMessage(FastaWriterOptions)


# @@protoc_insertion_point(module_scope)
