# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nucleus/protos/reference.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nucleus.protos import range_pb2 as nucleus_dot_protos_dot_range__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='nucleus/protos/reference.proto',
  package='nucleus.genomics.v1',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1enucleus/protos/reference.proto\x12\x13nucleus.genomics.v1\x1a\x1anucleus/protos/range.proto\"\xbf\x01\n\nContigInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0f\n\x07n_bases\x18\x03 \x01(\x03\x12\x39\n\x05\x65xtra\x18\x05 \x03(\x0b\x32*.nucleus.genomics.v1.ContigInfo.ExtraEntry\x12\x14\n\x0cpos_in_fasta\x18\x04 \x01(\x05\x1a,\n\nExtraEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"N\n\x11ReferenceSequence\x12*\n\x06region\x18\x01 \x01(\x0b\x32\x1a.nucleus.genomics.v1.Range\x12\r\n\x05\x62\x61ses\x18\x02 \x01(\tb\x06proto3')
  ,
  dependencies=[nucleus_dot_protos_dot_range__pb2.DESCRIPTOR,])




_CONTIGINFO_EXTRAENTRY = _descriptor.Descriptor(
  name='ExtraEntry',
  full_name='nucleus.genomics.v1.ContigInfo.ExtraEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='nucleus.genomics.v1.ContigInfo.ExtraEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='nucleus.genomics.v1.ContigInfo.ExtraEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=231,
  serialized_end=275,
)

_CONTIGINFO = _descriptor.Descriptor(
  name='ContigInfo',
  full_name='nucleus.genomics.v1.ContigInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='nucleus.genomics.v1.ContigInfo.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='description', full_name='nucleus.genomics.v1.ContigInfo.description', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='n_bases', full_name='nucleus.genomics.v1.ContigInfo.n_bases', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extra', full_name='nucleus.genomics.v1.ContigInfo.extra', index=3,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pos_in_fasta', full_name='nucleus.genomics.v1.ContigInfo.pos_in_fasta', index=4,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTIGINFO_EXTRAENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=84,
  serialized_end=275,
)


_REFERENCESEQUENCE = _descriptor.Descriptor(
  name='ReferenceSequence',
  full_name='nucleus.genomics.v1.ReferenceSequence',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='region', full_name='nucleus.genomics.v1.ReferenceSequence.region', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bases', full_name='nucleus.genomics.v1.ReferenceSequence.bases', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=277,
  serialized_end=355,
)

_CONTIGINFO_EXTRAENTRY.containing_type = _CONTIGINFO
_CONTIGINFO.fields_by_name['extra'].message_type = _CONTIGINFO_EXTRAENTRY
_REFERENCESEQUENCE.fields_by_name['region'].message_type = nucleus_dot_protos_dot_range__pb2._RANGE
DESCRIPTOR.message_types_by_name['ContigInfo'] = _CONTIGINFO
DESCRIPTOR.message_types_by_name['ReferenceSequence'] = _REFERENCESEQUENCE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ContigInfo = _reflection.GeneratedProtocolMessageType('ContigInfo', (_message.Message,), {

  'ExtraEntry' : _reflection.GeneratedProtocolMessageType('ExtraEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTIGINFO_EXTRAENTRY,
    '__module__' : 'nucleus.protos.reference_pb2'
    # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.ContigInfo.ExtraEntry)
    })
  ,
  'DESCRIPTOR' : _CONTIGINFO,
  '__module__' : 'nucleus.protos.reference_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.ContigInfo)
  })
_sym_db.RegisterMessage(ContigInfo)
_sym_db.RegisterMessage(ContigInfo.ExtraEntry)

ReferenceSequence = _reflection.GeneratedProtocolMessageType('ReferenceSequence', (_message.Message,), {
  'DESCRIPTOR' : _REFERENCESEQUENCE,
  '__module__' : 'nucleus.protos.reference_pb2'
  # @@protoc_insertion_point(class_scope:nucleus.genomics.v1.ReferenceSequence)
  })
_sym_db.RegisterMessage(ReferenceSequence)


_CONTIGINFO_EXTRAENTRY._options = None
# @@protoc_insertion_point(module_scope)
